@@ -1,3 +1,7 @@
+""" The lane module contains the basic classes for Lanes and roadmarks
+
+"""
+
 import xml.etree.ElementTree as ET
 from .helpers import enum2str
 from .enumerations import LaneType, LaneChange, RoadMarkWeight, RoadMarkColor, RoadMarkType, MarkRule 
@@ -19,12 +23,17 @@ class Lanes():
 
             add_lanesection(lanesection)
                 adds a lane section to Lanes
+
+            add_laneoffset(laneoffset) 
+                adds a lane offset to Lanes
     """
     def __init__(self):
         """ initalize Lanes
 
         """
         self.lanesections = []
+        self.laneoffsets = []
+
     def add_lanesection(self,lanesection, lanelinks=None):
         """ creates the Lanes element of opendrive
     
@@ -51,6 +60,17 @@ class Lanes():
           
         self.lanesections.append(lanesection)
 
+    def add_laneoffset(self, laneoffset):
+        """ adds a lane offset to Lanes
+
+        Parameters
+        ----------
+            laneoffset (LaneOffset): a LaneOffset to add
+        """
+        if not isinstance(laneoffset, LaneOffset):
+            raise TypeError('add_laneoffset requires a LaneOffset as input, not ' + str(type(laneoffset)))
+        self.laneoffsets.append(laneoffset)
+
     def get_element(self):
         """ returns the elementTree of Lanes
 
@@ -58,6 +78,8 @@ class Lanes():
         element = ET.Element('lanes')
         for l in self.lanesections:
             element.append(l.get_element())
+        for l in self.laneoffsets:
+            element.append(l.get_element())
         return element
 
 class LaneSection():
@@ -164,6 +186,82 @@ class LaneSection():
 
         return element
         
+class LaneOffset():
+    """ the LaneOffset class defines an overall lateral offset along the road, described as a third degree polynomial
+
+        Parameters
+        ----------
+            s (float): s start coordinate of the elevation
+
+            a (float): a coefficient of the polynomial
+
+            b (float): b coefficient of the polynomial
+
+            c (float): c coefficient of the polynomial
+
+            d (float): d coefficient of the polynomial
+
+        Attributes
+        ----------
+            s (float): s start coordinate of the elevation
+
+            a (float): a coefficient of the polynomial
+
+            b (float): b coefficient of the polynomial
+
+            c (float): c coefficient of the polynomial
+
+            d (float): d coefficient of the polynomial
+
+        Methods
+        -------
+            get_element(elementname)
+                Returns the full ElementTree of the class
+
+            get_attributes()
+                Returns the attributes of the class
+
+    """
+    def __init__(self, s=0, a=0, b=0, c=0, d=0):
+        """ initalize the LaneOffset class
+
+        Parameters
+        ----------
+            s (float): s start coordinate of the LaneOffset
+
+            a (float): a coefficient of the polynomial
+
+            b (float): b coefficient of the polynomial
+
+            c (float): c coefficient of the polynomial
+
+            d (float): d coefficient of the polynomial
+
+        """
+        self.s = s
+        self.a = a
+        self.b = b
+        self.c = c
+        self.d = d
+
+    def get_attributes(self):
+        """ returns the attributes of the LaneOffset
+        """
+    
+        retdict = {}
+        retdict['s'] = str(self.s)
+        retdict['a'] = str(self.a)
+        retdict['b'] = str(self.b)
+        retdict['c'] = str(self.c)
+        retdict['d'] = str(self.d)
+        return retdict
+
+    def get_element(self):
+        """ returns the elementTree of the LaneOffset
+        """
+        element = ET.Element('laneOffset',attrib=self.get_attributes())
+        
+        return element
 
 class Lane():
     """ creates a Lane of opendrive
@@ -234,19 +332,19 @@ class Lane():
             lane_type (LaneType): type of lane
                 Default: LaneType.driving
 
-            a (float): a coefficient
+            a (float): a polynomial coefficient for width (left/right) or laneoffset (center)
                 Default: 0
 
-            b (float): b coefficient
+            b (float): b polynomial coefficient for width (left/right) or laneoffset (center)
                 Default: 0
 
-            c (float): c coefficient
+            c (float): c polynomial coefficient for width (left/right) or laneoffset (center)
                 Default: 0
 
-            d (float): d coefficient
+            d (float): d polynomial coefficient for width (left/right) or laneoffset (center)
                 Default: 0
 
-            soffset (float): soffset of lane
+            soffset (float): soffset of lane renamed to s in case of centerlane
                 Default: 0
 
         """ 
@@ -276,10 +374,12 @@ class Lane():
         self.links.add_link(_Link(link_type,str(id)))
 
     def _set_lane_id(self,lane_id):
-        """ set the lane id of the lane
+        """ set the lane id of the lane and set lane type to 'none' in case of centerlane
 
         """
         self.lane_id = lane_id
+        if self.lane_id == 0:
+            self.lane_type = LaneType.none
 
     def add_roadmark(self,roadmark):
         """ add_roadmark adds a roadmark to the lane
@@ -334,16 +434,25 @@ class Lane():
 
         """
         element = ET.Element('lane',attrib=self.get_attributes())
-        element.append(self.links.get_element())
-
-        widthdict = {}
-        widthdict['a'] = str(self.a)
-        widthdict['b'] = str(self.b)
-        widthdict['c'] = str(self.c)
-        widthdict['d'] = str(self.d)
-        widthdict['sOffset'] = str(self.soffset)
-
-        ET.SubElement(element,'width',attrib=widthdict)
+                
+        #polynomial dict either for width (left/right lanes) or laneOffset (center lane)
+        polynomialdict = {}
+        polynomialdict['a'] = str(self.a)
+        polynomialdict['b'] = str(self.b)
+        polynomialdict['c'] = str(self.c)
+        polynomialdict['d'] = str(self.d)
+        polynomialdict['sOffset'] = str(self.soffset) 
+        
+        #according to standard if lane is centerlane it should 
+        #not have a width record and omit the link record        
+        if self.lane_id != 0: 
+            element.append(self.links.get_element())
+            ET.SubElement(element,'width',attrib=polynomialdict)        
+        #use polynomial dict for laneOffset in case of center lane (only if values provided)
+        elif any([self.a,self.b,self.c,self.d]):
+            polynomialdict['s'] = polynomialdict.pop('sOffset')            
+            ET.SubElement(element,'laneOffset',attrib=polynomialdict)                         
+            
         if self.roadmark:
             element.append(self.roadmark.get_element())
             
