@@ -1,6 +1,9 @@
+""" the links module contains the basic classes and helpers for lane linking and junctions
+
+"""
 import xml.etree.ElementTree as ET
 from .helpers import enum2str
-from .enumerations import ElementType
+from .enumerations import ElementType, JunctionGroupType
 
 from .exceptions import NotSameAmountOfLanesError
 
@@ -213,6 +216,7 @@ class _Link():
 
 class LaneLinker():
     """ LaneLinker stored information for linking lane sections 
+        NOTE: Not part of OpenDRIVE, but a helper to link lanes for the user.
         
         Parameters
         ----------
@@ -264,7 +268,7 @@ class Connection():
         
         Parameters
         ----------
-            incoming_road (int): the id of the incoming road to the junciton
+            incoming_road (int): the id of the incoming road to the junction
 
             connecting_road (int): id of the connecting road (type junction)
             
@@ -274,7 +278,7 @@ class Connection():
 
         Attributes
         ----------
-            incoming_road (int): the id of the incoming road to the junciton
+            incoming_road (int): the id of the incoming road to the junction
 
             connecting_road (int): id of the connecting road (type junction)
             
@@ -301,7 +305,7 @@ class Connection():
 
         Parameters
         ----------
-            incoming_road (int): the id of the incoming road to the junciton
+            incoming_road (int): the id of the incoming road to the junction
 
             connecting_road (int): id of the connecting road (type junction)
             
@@ -435,6 +439,17 @@ class Junction():
 from .enumerations import ContactPoint
 
 def are_roads_consecutive(road1, road2): 
+    """ checks if road2 follows road1
+
+        Parameters
+        ----------
+            road1 (Road): the first road
+
+            road1 (Road): the second road
+        Returns
+        -------
+            bool
+    """
 
     if road1.successor is not None and road2.predecessor is not None: 
         if road1.successor.element_type == ElementType.road and road2.predecessor.element_type == ElementType.road:
@@ -595,3 +610,80 @@ def _create_links_roads(pre_road,suc_road):
     else:
         raise NotSameAmountOfLanesError('Road ' + str(pre_road.id) + ' and road ' + str(suc_road.id) + ' does not have the same number of right lanes.')
 
+class JunctionGroup():
+    """ JunctionGroup creates a JunctionGroup of OpenDRIVE
+        
+        Parameters
+        ----------
+            name (str): name of the junctiongroup
+
+            group_id (int): id of the junctiongroup
+
+            junction_type (JunctionGroupType): type of junction
+                Default: JunctionGroupType.roundabout
+
+        Attributes
+        ----------
+            name (str): name of the junctiongroup
+
+            group_id (int): id of the junctiongroup
+
+            junctions (list of int): all the junctions in the junctiongroup
+
+        Methods
+        -------
+            get_element()
+                Returns the full ElementTree of the class
+
+            get_attributes()
+                Returns a dictionary of all attributes of the class
+
+            add_junction(junction_id)
+                Adds a connection to the junction
+    """
+
+    def __init__(self,name,group_id,junction_type=JunctionGroupType.roundabout):
+        """ initalize the JunctionGroup
+
+        Parameters
+        ----------
+            name (str): name of the junctiongroup
+
+            group_id (int): id of the junctiongroup
+
+            junction_type (JunctionGroupType): type of junction
+                Default: JunctionGroupType.roundabout
+        """
+        self.name = name
+        self.group_id = group_id
+        self.junctions = []
+        self.junction_type = junction_type
+
+    def add_junction(self,junction_id):
+        """ Adds a new link to the Junction
+
+            Parameters
+            ----------
+                junction_id (int): adds a junction to the junctiongroup
+
+        """
+        self.junctions.append(junction_id)
+
+    def get_attributes(self):
+        """ returns the attributes as a dict of the Junction
+
+        """
+        retdict = {}
+        retdict['name'] = self.name
+        retdict['id'] = str(self.group_id)
+        retdict['type'] = enum2str(self.junction_type)
+        return retdict
+
+    def get_element(self):
+        """ returns the elementTree of the Junction
+
+        """
+        element = ET.Element('junctionGroup',attrib=self.get_attributes())
+        for j in self.junctions:
+            ET.SubElement(element,'junctionReference',attrib={'junction':str(j)})
+        return element
\ No newline at end of file
