@@ -1,9 +1,14 @@
+""" thte opendrive module contains the top level opendrive class aswell as the road class
+
+"""
 import xml.etree.ElementTree as ET
 
-from .helpers import printToFile
+from .helpers import printToFile,enum2str
 from .links import _Link, _Links, create_lane_links
 from .enumerations import ElementType, ContactPoint, RoadSide
 from .exceptions import UndefinedRoadNetwork, RoadsAndLanesNotAdjusted
+from .elevation import LateralProfile, ElevationProfile, _Poly3Profile
+
 
 import datetime as dt
 import warnings
@@ -17,10 +22,18 @@ class _Header():
         Parameters
         ----------
             name (str): name of the road 
+            
+            revMajor (str): major revision of OpenDRIVE
+                    
+            revMinor (str): minor revision of OpenDRIVE
 
         Attributes
         ----------
             name (str): name of the scenario 
+            
+            revMajor (str): major revision of OpenDRIVE
+                    
+            revMinor (str): minor revision of OpenDRIVE
 
         Methods
         -------
@@ -31,14 +44,21 @@ class _Header():
                 Returns a dictionary of all attributes of FileHeader
 
     """
-    def __init__(self,name):
+    def __init__(self, name, revMajor, revMinor):
         """ Initalize the Header
 
          Parameters
         ----------
             name (str): name of the road 
+            
+            revMajor (str): major revision of OpenDRIVE
+                    
+            revMinor (str): minor revision of OpenDRIVE
+                    
         """
         self.name = name
+        self.revMajor = revMajor
+        self.revMinor = revMinor
 
         
 
@@ -48,8 +68,8 @@ class _Header():
         """
         retdict = {}
         retdict['name'] = self.name
-        retdict['revMajor'] ='1'
-        retdict['revMinor'] ='5'
+        retdict['revMajor'] = str(self.revMajor)
+        retdict['revMinor'] = str(self.revMinor)
         retdict['date'] = str(dt.datetime.now())
         retdict['north'] = '0.0'
         retdict['south'] = '0.0'
@@ -97,14 +117,19 @@ class Road():
             road_type (int): type of road (junction)
                 Default: -1
 
-            name (str): name of the road (optional)
+            name (str): name of the road
 
-            rule (TrafficRule): traffic rule (optional)
-
-            signals (Signal): Contains a list of Signal objects (optional)
+            rule (TrafficRule): traffic rule 
             
-            objects (Object): Contains a list of Object objects (optional)
+            signals (Signal): Contains a list of Signal objects 
+            
+            objects (Object): Contains a list of Object objects
 
+            types (list of _Type): contans a list or _Type objects (optional)
+
+            elevationprofile (ElevationProfile): the elevation profile of the road
+
+            lateralprofile (LateralProfile): the lateral profile of the road
         Methods
         -------
             get_element()
@@ -113,9 +138,35 @@ class Road():
             get_attributes()
                 Returns a dictionary of all attributes of the class
 
-            write_xml(filename)
-                write a open scenario xml
-                
+            add_successor (element_type,element_id,contact_point,lane_offset)
+                adds a successor for the road
+
+            add_predecessor (element_type,element_id,contact_point,lane_offset)
+                adds a predecessor for the road          
+
+            add_neighbor (element_type,element_id,direction)
+                adds a neighbor for the road   
+
+            add_object (road_object)
+                adds an object to the road
+
+            add_elevation(s,a,b,c,d)
+                adds an elevation profile to the road
+
+            add_superelevation(s,a,b,c,d) 
+                adds a superelevation to the road
+
+            add_shape(s,t,a,b,c,d,e)
+                adds a lateral shape to the road
+            
+            add_object_roadside (road_object_prototype, repeatDistance, sOffset=0, tOffset=0, side=RoadSide.both)
+                adds an repeated object to the road
+
+            add_signal (signal)
+                adds a signal to the road
+
+            get_end_point ()
+                returns the x, y and heading at the end of the road
     """
     def __init__(self,road_id,planview,lanes, road_type = -1,name=None, rule=None):
         """ initalize the Road
@@ -151,6 +202,9 @@ class Road():
         self.adjusted = False
         self.objects = []
         self.signals = []
+        self.types = []
+        self.elevationprofile = ElevationProfile()
+        self.lateralprofile = LateralProfile()
 
     def add_successor(self,element_type,element_id,contact_point=None,lane_offset=0):
         """ add_successor adds a successor link to the road
@@ -207,6 +261,60 @@ class Road():
     
         self.links.add_link(suc)
         self._neighbor_added += 1
+
+    def add_elevation(self,s,a,b,c,d):
+        """ ads an elevation profile to the road (3-degree polynomial)
+
+            Parameters
+            ----------
+                s (float): s start coordinate of the elevation
+
+                a (float): a coefficient of the polynomial
+
+                b (float): b coefficient of the polynomial
+
+                c (float): c coefficient of the polynomial
+
+                d (float): d coefficient of the polynomial
+        """
+        self.elevationprofile.add_elevation(_Poly3Profile(s,a,b,c,d))
+
+    def add_superelevation(self,s,a,b,c,d):
+        """ ads a superelevation profile to the road (3-degree polynomial)
+
+            Parameters
+            ----------
+                s (float): s start coordinate of the superelevation
+
+                a (float): a coefficient of the polynomial
+
+                b (float): b coefficient of the polynomial
+
+                c (float): c coefficient of the polynomial
+
+                d (float): d coefficient of the polynomial
+        """
+        self.lateralprofile.add_superelevation(_Poly3Profile(s,a,b,c,d))
+    
+    def add_shape(self,s,t,a,b,c,d):
+        """ ads a superelevation profile to the road (3-degree polynomial)
+
+            Parameters
+            ----------
+                s (float): s start coordinate of the superelevation
+
+                t (flaot): the t start coordinate of the lateral profile
+
+                a (float): a coefficient of the polynomial
+
+                b (float): b coefficient of the polynomial
+
+                c (float): c coefficient of the polynomial
+
+                d (float): d coefficient of the polynomial
+        """
+        self.lateralprofile.add_shape(_Poly3Profile(s,a,b,c,d,t))
+    
     def add_object(self,road_object):
         """ add_object adds an object to a road and calls a function that ensures unique IDs
         
@@ -245,7 +353,7 @@ class Road():
         """
         if not self.planview.adjusted:
             raise RoadsAndLanesNotAdjusted("Could not add roadside object because roads and lanes need to be adjusted first. Consider calling 'adjust_roads_and_lanes()'.")
-            return
+
         
         hdg_factors = []
         total_widths = []
@@ -289,7 +397,24 @@ class Road():
             signal._update_id()
             self.signals.append(signal)
 
-        
+    def add_type(self,road_type,s = 0,country=None,speed=None,speed_unit='m/s'):
+        """ adds a type to the road (not to mix with junction or not as the init)
+
+            Parameters
+            ----------
+                road_type (RoadType): the type of road
+
+                s (float): the distance where it starts
+                    Default: 0
+
+                country (str): country code (should follow ISO 3166-1,alpha-2) (optional)
+
+                speed (float/str): the maximum speed allowed
+
+                sped_unit (str): unit of the speed, can be 'm/s','mph,'kph'
+        """
+        self.types.append(_Type(road_type,s,country,speed,speed_unit))
+
         
     def get_end_point(self):
         """ get the x, y, and heading, of the end of the road
@@ -322,6 +447,9 @@ class Road():
         """
         element = ET.Element('road',attrib=self.get_attributes())
         element.append(self.links.get_element())
+        if self.types:
+            for r in self.types:
+                element.append(r.get_element())
         element.append(self.planview.get_element())
         element.append(self.lanes.get_element())
         if len(self.signals) > 0:
@@ -332,7 +460,8 @@ class Road():
             objectselement = ET.SubElement(element,'objects')
             for road_object in self.objects:
                 objectselement.append(road_object.get_element())        
-            
+        element.append(self.elevationprofile.get_element())
+        element.append(self.lateralprofile.get_element())
         return element
 
 class OpenDrive():
@@ -341,10 +470,22 @@ class OpenDrive():
         Parameters
         ----------
             name (str): name of the road
+            
+            revMajor (str): major revision of OpenDRIVE written to header
+                Default: '1'
+                    
+            revMinor (str): minor revision of OpenDRIVE written to header
+                Default: '5'
 
         Attributes
         ----------
             name (str): name of the road
+            
+            revMajor (str): major revision of OpenDRIVE written to header
+                Default: '1'
+                    
+            revMinor (str): minor revision of OpenDRIVE written to header
+                Default: '5'
 
             roads (list of Road): all roads 
 
@@ -365,7 +506,7 @@ class OpenDrive():
                 write a open scenario xml
                 
     """
-    def __init__(self,name):
+    def __init__(self, name, revMajor='1', revMinor='5'):
         """ Initalize the Header
 
             Parameters
@@ -374,7 +515,9 @@ class OpenDrive():
 
         """
         self.name = name
-        self._header = _Header(self.name)
+        self.revMajor = revMajor
+        self.revMinor = revMinor
+        self._header = _Header(self.name, self.revMajor, self.revMinor)
         self.roads = {}
         self.junctions = []
         #self.road_ids = []
@@ -437,6 +580,9 @@ class OpenDrive():
                 h = h + np.pi #we are attached to the predecessor's start, so road[k] will start in its opposite direction 
             elif contact_point == ContactPoint.end:
                 x,y,h = self.roads[str(neighbour_id)].planview.get_end_point()
+            else:
+                raise ValueError('Unknown ContactPoint')
+
             num_lane_offsets = main_road.lane_offset_pred
             x = -num_lane_offsets*3*np.sin(h) + x
             y = num_lane_offsets*3*np.cos(h) + y
@@ -451,6 +597,8 @@ class OpenDrive():
                 h = h + np.pi #we are attached to the predecessor's start, so road[k] will start in its opposite direction 
             elif contact_point == ContactPoint.end:
                 x,y,h = self.roads[str(neighbour_id)].planview.get_end_point()
+            else:
+                raise ValueError('Unknown ContactPoint')
             num_lane_offsets = main_road.lane_offset_suc
             x = num_lane_offsets*3*np.sin(h) + x
             y = -num_lane_offsets*3*np.cos(h) + y
@@ -477,19 +625,21 @@ class OpenDrive():
         count_total_adjusted_roads = 0
         fixed_road = False
         for k in self.roads:
-            if self.roads[k].planview.fixed:
+            if self.roads[k].planview.fixed and not self.roads[k].planview.adjusted:
                 self.roads[k].planview.adjust_geometries()
                 # print('Fixing Road: ' + k)
                 count_total_adjusted_roads += 1
                 fixed_road = True
-
+            elif self.roads[k].planview.adjusted:
+                fixed_road = True
+                count_total_adjusted_roads += 1
 
         # If no roads are fixed, select the first road is selected as the pivot-road
-        if fixed_road is False: 
-            self.roads[list(self.roads.keys())[0]].planview.adjust_geometries()
-            # print('Selecting and adjusting the first road {}'.format(self.roads[list(self.roads.keys())[0] ].id))
-            count_total_adjusted_roads += 1
-            
+        if len(self.roads) > 0:
+            if fixed_road is False: 
+                self.roads[list(self.roads.keys())[0]].planview.adjust_geometries()
+                # print('Selecting and adjusting the first road {}'.format(self.roads[list(self.roads.keys())[0] ].id))
+                count_total_adjusted_roads += 1
         
         while count_total_adjusted_roads < len(self.roads):
             
@@ -590,4 +740,85 @@ class OpenDrive():
         if filename == None:
             filename = self.name + '.xodr'
         printToFile(self.get_element(),filename,prettyprint)
-        
\ No newline at end of file
+        
+class _Type():
+    """ class to generate the type element of a road, (not the Enumeration it self).
+
+        Parameters
+        ----------
+            road_type (RoadType): the type of road
+
+            s (float): the distance where it starts
+                Default: 0
+
+            country (str): country code (should follow ISO 3166-1,alpha-2) (optional)
+
+            speed (float/str): the maximum speed allowed
+
+            speed_unit (str): unit of the speed, can be 'm/s','mph,'kph'
+
+        Attributes
+        ----------
+            road_type (RoadType): the type of road
+
+            s (float): the distance where it starts
+
+            country (str): country code (should follow ISO 3166-1,alpha-2) (optional)
+
+            speed (float/str): can either be a float or the following strings: "no limit" or "undefined"
+
+            speed_unit (str): unit of the speed
+    """
+    def __init__(self,road_type,s=0,country=None,speed=None,speed_unit='m/s'):
+        """ initalize the _Type
+
+        Parameters
+        ----------
+            road_type (RoadType): the type of road
+
+            s (float): the distance where it starts
+                Default: 0
+
+            country (str): country code (should follow ISO 3166-1,alpha-2) (optional)
+
+            speed (float/str): the maximum speed allowed
+
+            speed_unit (str): unit of the speed, can be 'm/s','mph,'kph'
+
+
+        """
+        self.road_type = road_type
+        self.s = s
+        self.country = country
+        if isinstance(speed,float) or isinstance(speed,int) or speed in ['no limit', 'undefined'] or speed == None:
+            self.speed = speed
+        else:
+            if isinstance(speed,str):
+                raise ValueError('speed can only be numerical or "no limit" and "undefined", not: ' + str(speed_unit))
+            
+        if speed_unit not in ['m/s','mph','kph']:
+            raise ValueError('speed_unit can only be m/s, mph, or kph, not: ' + speed_unit)
+        self.speed_unit = speed_unit
+
+
+    def get_attributes(self):
+        """ returns the attributes as a dict of the _Type
+
+        """
+        retdict = {}
+
+        retdict['s'] = str(self.s)
+        retdict['type'] = enum2str(self.road_type)
+        if self.country:
+            retdict['country'] = self.country
+        return retdict
+
+    def get_element(self):
+        """ returns the elementTree of the _Type
+
+        """
+
+        element = ET.Element('type',attrib=self.get_attributes())
+        if self.speed:
+            ET.SubElement(element,'speed',attrib={'max':str(self.speed),'unit':self.speed_unit})
+        return element
\ No newline at end of file
