@@ -1,5 +1,7 @@
 # __init__.py
+""" the xosc module contains the classes used to generate OpenSCENARIO (xml) files. 
 
+"""
 from .utils import *
 from .actions import *
 from .helpers import *
@@ -8,4 +10,5 @@ from .triggers import *
 from .scenario import *
 from .storyboard import *
 from .entities import *
-from .enumerations import *
\ No newline at end of file
+from .enumerations import *
+from .catalog_reader import *
\ No newline at end of file
